
;--------------------------------------------
;mc_neutroni_driver.pro
;--------------------------------------------
;syksy 2007 simulaatiokurssi (HS)
;neutronien ampuminen levyyn
;vertaa eri menetelmien virheita eri N arvolla

;menetelma
;1 neutronin rata paattyy absorbtioon, heijastukseen tai lapaisyyn
;2 painomenetelma: absorbtio otetaan huomioon painoa pienentamalla
;                  rata paattyy heijastukseen tai lapaisyyn
;3 painometelma: jokainen ratasegmentti pakotetaan sirontaa
;                absorbtio, heijatus ja lapaisy otetaan huomioon
;                painoa pienetamalla
;                



;parametrit 
;levyn paksuus h (suhteessa keskim.vapaaseen matkaan)
;alb=sironnan vaik.ala/kokonais vaik.ala

h=2.
alb=1.
method=3
program='mc_neutroni_driver_method3_h20_alb10'
ps=2
psdirect,program,ps

;tutkittavat N
ntab=[1d1,1d2,1d3,1d4]

;taulukot: tallennetaan eri N:arvojen tulokset
;esim lapaisy:
;  p_lap  = lapaisy tn.
; dp_lap  = lapaisy tn keskivirhe kerattyna MC-ohjelmassa
; dp_lap2 = lapaisy tn keskivirhe arvioituna k=20 toiston otoksesta

p_lap_n=ntab*0.
dp_lap_n=ntab*0.
dp_lap_n2=ntab*0.

p_abs_n=ntab*0.
dp_abs_n=ntab*0.
dp_abs_n2=ntab*0.

p_hei_n=ntab*0.
dp_hei_n=ntab*0.
dp_hei_n2=ntab*0.



for in=0,n_elements(ntab)-1 do begin
   n=ntab(in)
   
   K=20   
;aputaulukko, johon tallennetaan tulokset K toistosta
   p_lap_tab=fltarr(k)
   p_abs_tab=fltarr(k)
   p_hei_tab=fltarr(k)

   
   seed=0

   for ik=0,k-1 do begin
      if(method eq 1) then begin
         mc_neutroni_method1_f,$
            h,alb,n, p_lap,p_abs,p_hei, dp_lap,dp_abs,dp_hei,seed=seed
      endif
      if(method eq 2) then begin
         mc_neutroni_method2_f,$
            h,alb,n, p_lap,p_abs,p_hei, dp_lap,dp_abs,dp_hei,seed=seed
      endif
      if(method eq 3) then begin
         mc_neutroni_method3_f,$
            h,alb,n, p_lap,p_abs,p_hei, dp_lap,dp_abs,dp_hei,seed=seed
      endif

      p_lap_tab(ik)=p_lap
      p_abs_tab(ik)=p_abs
      p_hei_tab(ik)=p_hei
      
      if(ik mod (k/5.) eq 0) then print,ik
   endfor
   
   print,'N=',n
   print,'LAPAISY:  ',mean(p_lap_tab),stdev(p_lap_tab),dp_lap
   print,'ABSORBTIO:',mean(p_abs_tab),stdev(p_abs_tab),dp_abs
   print,'HEIJASTUS:',mean(p_hei_tab),stdev(p_hei_tab),dp_hei
   
   p_lap_n(in)=mean(p_lap_tab)
   dp_lap_n(in)=stdev(p_lap_tab)
   dp_lap_n2(in)=dp_lap

   p_abs_n(in)=mean(p_abs_tab)
   dp_abs_n(in)=stdev(p_abs_tab)
   dp_abs_n2(in)=dp_abs

   p_hei_n(in)=mean(p_hei_tab)
   dp_hei_n(in)=stdev(p_hei_tab)
   dp_hei_n2(in)=dp_hei

endfor

if(method eq 1) then smethod='met1'
if(method eq 2) then smethod='met2: painomenetelma'
if(method eq 3) then smethod='met3: pakotettu sironta'

sn='N='+string(n,'(i7)')
sha='H='+string(h,'(f6.2)')+'  ALB='+string(alb,'(f6.2)')
nwin

!p.multi=[0,3,1]
!p.charsize=2.
imax=n_elements(ntab)-1
ff8='(f8.5)'

plot,ntab,dp_lap_n,/ylog,/xlog,title=smethod,yr=[1d-6,1],ys=1,$
     ytitle='LAPAISY',xtitle='N'
oplot,ntab,dp_lap_n2,psym=6
oplot,ntab,1./sqrt(ntab),lines=2
xyouts,20,0.5,'p='+string(p_lap_n(imax),ff8),chars=1.

plot,ntab,dp_abs_n,/ylog,/xlog,title=sn,yr=[1d-6,1],ys=1,$
     ytitle='ABSORBTIO',xtitle='N'
oplot,ntab,dp_abs_n2,psym=6
oplot,ntab,1./sqrt(ntab),lines=2
xyouts,20,0.5,'p='+string(p_abs_n(imax),ff8),chars=1.

plot,ntab,dp_hei_n,/ylog,/xlog,title=sha,yr=[1d-6,1],ys=1,$
     ytitle='HEIJASTUS',xtitle='N'
oplot,ntab,dp_hei_n2,psym=6
oplot,ntab,1./sqrt(ntab),lines=2
xyouts,20,0.5,'p='+string(p_hei_n(imax),ff8),chars=1.

psdirect,program,ps,/stop
!p.charsize=1
end
