
pro mc_neutroni_method1_f,H,ALB,N,p_lap,p_abs,p_hei,dp_lap,dp_abs,dp_hei,$
                  print=print,plot=plot,oplot=oplot,seed0=seed0

if(n_params() le 0) then begin
   print,'----------------------------------------------------------------------------'
   print,'mc_neutroni_method1_f, H, ALB, N, p_lap, p_abs, p_hei, dp_lap, dp_abs, dp_hei'
   print,'----------------------------------------------------------------------------'
   print,''
   print,'Levyn neutronilapaisevyys: menetelma 1'
   print,'seurataan rataa kunnes se paattyy absorbtioon, heijastukseen tai lapaisyyn'
   print,'----------------------------------------------------------------------------'
   print,''
   print,'INPUT-parametrit'
   print,'  H   = paksuus/keskim.vapaa matka'
   print,'  ALB = sironta/kok.vaik.ala'
   print,'OUTPUT:'
   print,' p_lap,  p_abs,  p_hei = lapaisy, absorbtio ja heijastus todennakoisyydet'
   print,'dp_lap, dp_abs, dp_hei = vastaavat keskivirheet'
   print,'KEYWORDIT:'
   print,'/print -> printtaa tulos'
   print,'/plot  -> nayta kukin ratahistoria yksi kerraallaan'
   print,'/oplot  -> nayta kaikki ratahistoriat'
   print,'seed=arvo  -> siirra seed kutsuvasta ohjelmasta ja sinne takaisin (TARKEA!)'
   print,''
   print,'ESIMERKKI: TARKISTUS'
   print,'  kutsun jalkeen paina ret-> uusi rata samaan ikkunaan'
   print,'  anna kutsu useita kertoja perakkain - mita huomaat?!'
   print,'mc_neutroni_method1_f,10.,1.,10,/oplot'
   print,'----------------------------------------------------------------------------'
   print,'Heikki Salo 31.10.2007   simulaatiot harj 3'
   print,'----------------------------------------------------------------------------'
   return
endif


;Parametrit
; levyn paksuus/vapaa matka H  (eli vapaa matka = 1 yksikko)
; Sigma_sca/Sigma_tot  =ALB
;laskettava heijastumis, absorbtio ja lapaisy todennakoisyydet



if(keyword_set(seed0)) then seed=seed0

;------------------------------------------------------------
;MENETELMA 1:
;------------------------------------------------------------
;seurataan i=1,..,N neutronin rataa kunnes se
;  heijastuu     -> lisays laskuriin W_hei=W_hei+1
;  absorpoituu   -> lisays laskuriin W_abs=W_abs+1
;  lapaisee      -> lisays laskuriin W_lap=W_lap+1

;vastaavat todennakoisyydet 
;  p_hei=W_hei/N
;  p_abs=W_abs/N
;  p_lap=W_lap/N

;virhearviot:
; Esim. heijastus
;  W_hei=  \sum_i^N g_i   g_i=1  heijastus, 0, muulloin
;  Dp_hei= \sqrt((<g^2> -<g>^2)/N)
;          jossa  <g>   = \sum g_i/N
;                 <g^2> = \sum {g_i}^2/N

;kerätään laskureita:
  W_hei=0.               ;\sum g_hei
  W_abs=0.               ;\sum g_abs
  W_lap=0.               ;\sum g_lap
  W_hei2=0.              ;\sum g_hei^2
  W_abs2=0.              ;\sum g_abs^2
  W_lap2=0.              ;\sum g_lap^2
  
;varmuuden vuoksi:
  kmax=1000                     ;sirontojen maksimi maara

  

;ratojen piirtamiseen liittyen
  if(keyword_set(plot) or keyword_set(oplot)) then nwin
  x_plot=fltarr(kmax)


;------------------------------------------------------------
;silmukka yli neutroniratojen
;aluksi x=0 ja suunta kohtisuoraan levyä vasten
;------------------------------------------------------------

  for i=0l,n-1 do begin

;ensimmaisen vuorovaikutuksen paikka
     eta=randomu(seed,1)
     lambda=-alog(eta)
     x=lambda
  
     x_plot(1)=x       ;mahdollista plottausta varten
     xmui=1            ;--"--

;lapaisee suoraan?
;lisataan lapaisy-laskureita ja 
;hypataan seuraavaan neutroniin
     if(x gt H) then begin
        w_lap=w_lap+1.
        w_lap2=w_lap2+1.
        goto,next
     endif

;jatketaan neutronin rataa
     for k=1,kmax do begin
              
;absorbtio ?
        eta=randomu(seed,1)
        if(eta gt alb) then begin
           w_abs=w_abs+1.
           w_abs2=w_abs2+1.
           goto,next
        endif
        
;sironnan jalkeinen suunta
        eta=randomu(seed,1)
        cosa=eta*2.-1.
        
;sironnan jalkeinen matka
        eta=randomu(seed,1)
        lambda=-alog(eta)
        
;seuraavan vuorovaikutuksen syvyys
        x=x+cosa*lambda
        x_plot(k+2)=x               ;mahdollista plottausta varten
        xmui=k+2                    ;--"--

;lapaisy?
        if(x ge h) then begin
           w_lap=w_lap+1.
           w_lap2=w_lap2+1.
           goto,next
        endif
        
;heijastus?
        if(x le 0) then begin
           w_hei=w_hei+1.
           w_hei2=w_hei2+1.
           goto,next
        endif
     endfor
     
;seuraavaan radanpatkaan   
next:

     
;PLOTTAUS?
;--------------------------------
;plotaan radat yksi kerrallaan?  
  if(keyword_set(plot)) then begin     
     plot,x_plot(0:xmui),yr=[-0.1*h,h*1.1],ys=1,psym=-6,syms=.2,$
          title='neutroni n:o '+string(i),xtitle='vuorovaikutus n:o',ytitle='x'
     oplot,x_plot*0,lines=1
     oplot,x_plot*0+h,lines=1

     vast=''
     read,vast
  endif


;--------------------------------
;plotataan kaikki radat? samaan ikkunaan?
  if(keyword_set(oplot)) then begin     
     if(i eq 0) then begin
        plot,x_plot(0:xmui),yr=[-0.1*h,h*1.1],xr=[0,150],$
             ys=1,psym=-6,syms=.2,$
             title='neutroni n:o '+string(i),$
             xtitle='vuorovaikutus n:o',ytitle='x'
        oplot,x_plot*0,lines=1
        oplot,x_plot*0+h,lines=1
     endif
     oplot,x_plot(0:xmui<100),col=i+2<30

     if(n le 20) then begin
        xyouts,110,h-h/20.*(i+1),x_plot(1),col=i+2
     endif
     vast=''
     read,vast
  endif

endfor   ;seuraava neutroni

;-----------------------------------------------------
;tulosten kokoaminen

  p_lap=w_lap/n
  p_abs=w_abs/n
  p_hei=w_hei/n

  sqrtn=sqrt(1.*n)
  dp_lap=sqrt(w_lap2/n-(w_lap/n)^2)/sqrtn
  dp_abs=sqrt(w_abs2/n-(w_abs/n)^2)/sqrtn
  dp_hei=sqrt(w_hei2/n-(w_hei/n)^2)/sqrtn

  
;-----------------------------------------------------
  if(keyword_set(print)) then begin
     print,'H,Alb,n',h,alb,n
     print,'plap,pabs,phei',p_lap,p_abs,p_hei
     print,'dlap,dabs,dhei',dp_lap,dp_abs,dp_hei
  endif
  
  seed0=seed
end
