pro mc_neutroni_method2_f,h,alb,n,p_lap,p_abs,p_hei,dp_lap,dp_abs,dp_hei,$
                  print=print,plot=plot,oplot=oplot,seed0=seed0

if(n_params() le 0) then begin
   print,'----------------------------------------------------------------------------'
   print,'mc_neutroni_method2_f, H, ALB, N, p_lap, p_abs, p_hei, dp_lap, dp_abs, dp_hei'
   print,'----------------------------------------------------------------------------'
   print,''
   print,'Levyn neutronilapaisevyys: menetelma 2'
   print,' Pakotetaan jokainen vuorovaikutus=sironta'
   print,' Absorbtio otetaan huomioon neutronikimpun painoa vahentamalla'
   print,' Rata paattyy joko lapaisyyn tai heijastukseen'
   print,'----------------------------------------------------------------------------'
   print,''
   print,'INPUT-parametrit'
   print,'  H   = paksuus/keskim.vapaa matka'
   print,'  ALB = sironta/kok.vaik.ala'
   print,'OUTPUT:'
   print,' p_lap,  p_abs,  p_hei = lapaisy, absorbtio ja heijastus todennakoisyydet'
   print,'dp_lap, dp_abs, dp_hei = vastaavat keskivirheet'
   print,'KEYWORDIT:'
   print,'/print -> printtaa tulos'
   print,'/plot  -> nayta kukin ratahistoria yksi kerraallaan'
   print,'/oplot  -> nayta kaikki ratahistoriat'
   print,'seed=arvo  -> siirra seed kutsuvasta ohjelmasta ja sinne takaisin (TARKEA!)'
   print,''
   print,'ESIMERKKI: TARKISTUS'
   print,'  kutsun jalkeen paina ret-> uusi rata samaan ikkunaan'
   print,'  anna kutsu useita kertoja perakkain - mita huomaat?!'
   print,'mc_neutroni_method2_f,10.,1.,10,/oplot'
   print,'----------------------------------------------------------------------------'
   print,'Heikki Salo 31.10.2007   simulaatiot harj 3'
   print,'----------------------------------------------------------------------------'
   return
endif

;Parametrit
; levyn paksuus/vapaa matka H  (eli vapaa matka = 1 yksikko)
; Sigma_sca/Sigma_tot  =ALB
;laskettava heijastumis, absorbtio ja lapaisy todennakoisyydet

if(keyword_set(seed0)) then seed=seed0

;------------------------------------------------------------
;MENETELMA 2:
;------------------------------------------------------------
;seurataan i=1,..,N neutronikimpun rataa kunnes se
;  heijastuu     -> lisays laskuriin W_hei=W_hei+W
;  lapaisee      -> lisays laskuriin W_lap=W_lap+W
;  W on neutronikimpun sen hetkinen paino (aluksi W=1)

;absorbtio kasitellaan pienentamalla neutronikimpun painoa
;  absorption tuoma lisa kimpun i painoon on summa yli ratasegmenttien
;  W_abs= W_abs + \sum W_abs_k

;vastaavat todennakoisyydet 
;  p_hei=W_hei/N
;  p_abs=W_abs/N
;  p_lap=W_lap/N

;virhearviot:
; Heijastus (lapaisy samlla tavalla)
;  W_hei=  \sum_i^N g_i   g_i=W  jos heijastuu, 0, muulloin
;  Dp_hei= \sqrt((<g^2> -<g>^2)/N)
;          jossa  <g>   = \sum g_i/N
;                 <g^2> = \sum {g_i}^2/N

;absorbtio samalla tavalla, paitsi etta  g_i=\sum W_abs_k

;kerataan laskureita:
  W_hei=0.               ;\sum g_hei
  W_abs=0.               ;\sum g_abs
  W_lap=0.               ;\sum g_lap
  W_hei2=0.              ;\sum g_hei^2
  W_abs2=0.              ;\sum g_abs^2
  W_lap2=0.              ;\sum g_lap^2
  
;tarvitaan myos erillinen laskuri keraamaan g_i yli k:n absorbtion
;w_abs_apu
  
  kmax=1000                     ;sirontojen maksimi maara
  wmin=1e-6                     ;lopetetaan rata kun W<Wmin  

;ratojen piirtamiseen liittyen
  if(keyword_set(plot) or keyword_set(oplot)) then nwin
  x_plot=fltarr(kmax)

;------------------------------------------------------------
;silmukka yli neutroniratojen
;aluksi x=0 ja suunta kohtisuoraan levya vasten
;------------------------------------------------------------


    
  for i=0l,n-1 do begin

;kunkin neutronikimpun alkuperainen paino
     weight=1.

;nollataan absorptioon liittyva apumuuttuja
     w_abs_apu=0.     

;ensimmaisen vuorovaikutuksen paikka
     eta=randomu(seed,1)
     lambda=-alog(eta)
     x=lambda
     x_plot(1)=x       ;mahdollista plottausta varten
     xmui=1            ;--"--



;lapaisee suoraan?
;lisataan lapaisy-laskureita ja 
;hypataan seuraavaan neutroniin

     if(x gt h) then begin
        w_lap=w_lap+weight
        w_lap2=w_lap2+weight^2
        goto,next
     endif

;jatketaan neutronikimpun rataa
     for k=1,kmax do begin

;lopetetaan jos paino pudonnut liian pieneksi?
        if(weight le wmin) then goto,next
                
;absorbtion kasittely (menetelma 2)
;pienennetaan kimpun painoa
;lisataan kimppuun i liittyvaan absorbtio-laskuria

        weight_abs=(1.-alb)*weight
        w_abs_apu=w_abs_apu+weight_abs
        weight=weight*alb
        
;sironnan jalkeinen suunta
        eta=randomu(seed,1)
        cosa=eta*2.-1.
        
;sironnan jalkeinen matka
        eta=randomu(seed,1)
        lambda=-alog(eta)


;seuraavan vuorovaikutuksen syvyys
        x=x+cosa*lambda
        x_plot(k+2)=x               ;mahdollista plottausta varten
        xmui=k+2                    ;--"--

;lapaisy?
        if(x ge h) then begin
           w_lap=w_lap+weight
           w_lap2=w_lap2+weight^2
           goto,next
        endif
        
;heijastus?
        if(x le 0) then begin
           w_hei=w_hei+weight
           w_hei2=w_hei2+weight^2
           goto,next
        endif
     endfor
     
;seuraavaan radanpatkaan   
next:

;lisataan neutronikimppuun i littyva painolaskuri varsinaisiin absorbtiolaskureihin
     w_abs=w_abs+w_abs_apu
     w_abs2=w_abs2+w_abs_apu^2




;PLOTTAUS?
;--------------------------------
;plotaan radat yksi kerrallaan?  
  if(keyword_set(plot)) then begin     
     plot,x_plot(0:xmui),yr=[-0.1*h,h*1.1],ys=1,psym=-6,syms=.2,$
          title='neutroni n:o '+string(i),xtitle='vuorovaikutus n:o',ytitle='x'
     oplot,x_plot*0,lines=1
     oplot,x_plot*0+h,lines=1

     vast=''
     read,vast
  endif


;--------------------------------
;plotataan kaikki radat? samaan ikkunaan?
  if(keyword_set(oplot)) then begin     
     if(i eq 0) then begin
        plot,x_plot(0:xmui),yr=[-0.1*h,h*1.1],xr=[0,150],$
             ys=1,psym=-6,syms=.2,$
             title='neutroni n:o '+string(i),$
             xtitle='vuorovaikutus n:o',ytitle='x'
        oplot,x_plot*0,lines=1
        oplot,x_plot*0+h,lines=1
     endif
     oplot,x_plot(0:xmui<100),col=i+2<30

     if(n le 20) then begin
        xyouts,110,h-h/20.*(i+1),x_plot(1),col=i+2
     endif
     vast=''
     read,vast
  endif

endfor   ;seuraava neutroni

;-----------------------------------------------------
;tulosten kokoaminen


  p_lap=w_lap/n
  p_abs=w_abs/n
  p_hei=w_hei/n

  sqrtn=sqrt(1.*n)
  dp_lap=sqrt(w_lap2/n-(w_lap/n)^2)/sqrtn
  dp_abs=sqrt(w_abs2/n-(w_abs/n)^2)/sqrtn
  dp_hei=sqrt(w_hei2/n-(w_hei/n)^2)/sqrtn

;-----------------------------------------------------
  if(keyword_set(print)) then begin
     print,'H,Alb,n',h,alb,n
     print,'plap,pabs,phei',p_lap,p_abs,p_hei
     print,'dlap,dabs,dhei',dp_lap,dp_abs,dp_hei
  endif



seed0=seed
end
