pro mc_neutroni_method3_f,h,alb,n,p_lap,p_abs,p_hei,dp_lap,dp_abs,dp_hei,$
                  print=print,plot=plot,oplot=oplot,seed0=seed0

if(n_params() le 0) then begin
   print,'----------------------------------------------------------------------------'
   print,'mc_neutroni_method3_f, H, ALB, N, p_lap, p_abs, p_hei, dp_lap, dp_abs, dp_hei'
   print,'----------------------------------------------------------------------------'
   print,''
   print,'Levyn neutronilapaisevyys: menetelma 3'
   print,''
   print,' Pakotetaan jokainen ratasegmentti sirontaan'
   print,' Absorbtio otetaan huomioon neutronikimpun painoa vahentamalla'
   print,' Samoin lapaisy ja heijastus'
   print,' Rata paattyy kun neutronikimpun paino alle 1e-6'
   print,'----------------------------------------------------------------------------'
   print,''
   print,'INPUT-parametrit'
   print,'  H   = paksuus/keskim.vapaa matka'
   print,'  ALB = sironta/kok.vaik.ala'
   print,'OUTPUT:'
   print,' p_lap,  p_abs,  p_hei = lapaisy, absorbtio ja heijastus todennakoisyydet'
   print,'dp_lap, dp_abs, dp_hei = vastaavat keskivirheet'
   print,'KEYWORDIT:'
   print,'/print -> printtaa tulos'
   print,'/plot  -> nayta kukin ratahistoria yksi kerraallaan'
   print,'/oplot  -> nayta kaikki ratahistoriat'
   print,'seed=arvo  -> siirra seed kutsuvasta ohjelmasta ja sinne takaisin (TARKEA!)'
   print,''
   print,'ESIMERKKI: TARKISTUS'
   print,'  kutsun jalkeen paina ret-> uusi rata samaan ikkunaan'
   print,'  anna kutsu useita kertoja perakkain - mita huomaat?!'
   print,'mc_neutroni_method2_f,10.,1.,10,/oplot'
   print,'----------------------------------------------------------------------------'
   print,'Heikki Salo 31.10.2007   simulaatiot harj 3'
   print,'----------------------------------------------------------------------------'
   return
endif

;Parametrit
; levyn paksuus/vapaa matka H  (eli vapaa matka = 1 yksikko)
; Sigma_sca/Sigma_tot  =ALB
;laskettava heijastumis, absorbtio ja lapaisy todennakoisyydet

if(keyword_set(seed0)) then seed=seed0

;------------------------------------------------------------
;MENETELMA 3:
;------------------------------------------------------------
;seurataan i=1,..,N neutronikimpun rataa kunnes sen paino on pudonnut
;alle ennalta annetun rajan (1e-6). Aluksi W=1.

;absorptio, heijastus ja lapaisy
;kasitellaan pienentamalla neutronikimpun painoa
;  absorption tuoma lisa kimpun i painoon on summa yli ratasegmenttien
;  W_abs= W_abs + \sum W_abs_k
;  heijastuksen tuoma lisa kimpun i painoon on summa yli ratasegmenttien
;  W_hei= W_hei + \sum W_hei_k
;  lapaisyn tuoma lisa kimpun i painoon on summa yli ratasegmenttien
;  W_lap= W_lap + \sum W_lap_k

;vastaavat todennakoisyydet 
;  p_hei=W_hei/N
;  p_abs=W_abs/N
;  p_lap=W_lap/N

;virhearviot:
; Heijastus (lapaisy ja absorbtio samalla tavalla)
;  W_hei=  \sum_i^N g_i   
;  g_i= \sum W_hei_k     jossa W_hei_k = heijastuksen paino k:n
;                                        vuorovaikutuksen jalkeen
;  Dp_hei= \sqrt((<g^2> -<g>^2)/N)
;          jossa  <g>   = \sum g_i/N
;                 <g^2> = \sum {g_i}^2/N

;kerataan laskureita:
  W_hei=0.               ;\sum g_hei
  W_abs=0.               ;\sum g_abs
  W_lap=0.               ;\sum g_lap
  W_hei2=0.              ;\sum g_hei^2
  W_abs2=0.              ;\sum g_abs^2
  W_lap2=0.              ;\sum g_lap^2
  
;tarvitaan myos erilliset laskurit keraamaan g_i yli k:n segmentin
;w_abs_apu
;w_hei_apu
;w_lap_apu

  
  kmax=1000                     ;sirontojen maksimi maara
  wmin=1e-6                     ;lopetetaan rata kun W<Wmin  

;ratojen piirtamiseen liittyen
  if(keyword_set(plot) or keyword_set(oplot)) then nwin
  x_plot=fltarr(kmax)

;------------------------------------------------------------
;silmukka yli neutroniratojen
;aluksi x=0 ja suunta kohtisuoraan levya vasten
;------------------------------------------------------------

  
  for i=0l,n-1 do begin

;kunkin neutronikimpun alkuperainen paino
     weight=1.
;nollataan kimppuun liittyvat apumuuttuja
     w_abs_apu=0.     
     w_lap_apu=0.
     w_hei_apu=0.
     
;ensimmaisen vuorovaikutuksen paikka
;pakotetaan valille 0,lambda_max=h

     eta=randomu(seed,1)
     lambda_max=h
     lambda=-alog(1.-eta+eta*exp(-lambda_max))
     x=lambda

;pienennetaan  kimpun painoa lapaisyn todennakoisyydella
;lisataan kimpun i lapaisy-laskuriin

     weight_lap=weight*exp(-lambda_max)
     weight=weight-weight_lap
     w_lap_apu=w_lap_apu+weight_lap
        
     x_plot(1)=x                ;mahdollista plottausta varten
     xmui=1                     ;--"--
     

;jatketaan neutronikimpun rataa
     for k=1,kmax do begin

;lopetetaan jos paino pudonnut liian pieneksi?
        if(weight le wmin) then goto,next
        

;absorbtion kasittely (sama kuin menetelmassa 2)
;pienennetaan kimpun painoa
;lisataan kimppuun i liittyvaan absorbtio-laskuria

        weight_abs=(1.-alb)*weight
        w_abs_apu=w_abs_apu+weight_abs
        weight=weight-weight_abs
        
;sironnan jalkeinen suunta
        eta=randomu(seed,1)
        cosa=eta*2.-1.

        if(cosa eq 0) then cosa=1d-6  ; joskus voi kayda nainkin?

;sironnan jalkeinen matka - riippuu suunnasta:

;       sironta alkuperaisessa suunnassa cosa>0 - lapaisy?
;       pienennetaan kimpun painoa
;       lisataan kimppuun i liittyvaa lapaisy-laskuria

        if(cosa gt 0) then begin
           lambda_max=(h-x)/cosa
           weight_lap=weight*exp(-lambda_max)
           w_lap_apu=w_lap_apu+weight_lap
           weight=weight-weight_lap
        endif

;       sironta takaisin pain  cosa<0 - heijastus?
;       pienennetaan kimpun painoa
;       lisataan kimppuun i liittyvaa heijastus-laskuria

        if(cosa lt 0) then begin
           lambda_max=-x/cosa
           weight_hei=weight*exp(-lambda_max)
           w_hei_apu=w_hei_apu+weight_hei
           weight=weight-weight_hei
        endif
        
;sironnan jalkeinen matka
        eta=randomu(seed,1)
        lambda=-alog(1.-eta+eta*exp(-lambda_max))

;seuraavan aironnan  syvyys
        x=x+cosa*lambda

        x_plot(k+2)=x               ;mahdollista plottausta varten
        xmui=k+2                    ;--"--

        
     endfor
     
;seuraavaan radanpatkaan   
next:

;lisataan neutronikimppuun i littyva painolaskuri varsinaisiin absorbtio.laskureihin
     w_abs=w_abs+w_abs_apu
     w_abs2=w_abs2+w_abs_apu^2

;lisataan neutronikimppuun i littyva painolaskuri varsinaisiin heijastus-laskureihin
     w_hei=w_hei+w_hei_apu
     w_hei2=w_hei2+w_hei_apu^2

;lisataan neutronikimppuun i littyva painolaskuri varsinaisiin lapaisy.laskureihin
     w_lap=w_lap+w_lap_apu
     w_lap2=w_lap2+w_lap_apu^2




;PLOTTAUS?
;--------------------------------
;plotaan radat yksi kerrallaan?  
  if(keyword_set(plot)) then begin     
     plot,x_plot(0:xmui),yr=[-0.1*h,h*1.1],ys=1,psym=-6,syms=.2,$
          title='neutroni n:o '+string(i),xtitle='vuorovaikutus n:o',ytitle='x'
     oplot,x_plot*0,lines=1
     oplot,x_plot*0+h,lines=1

     vast=''
     read,vast
  endif


;--------------------------------
;plotataan kaikki radat? samaan ikkunaan?
  if(keyword_set(oplot)) then begin     
     if(i eq 0) then begin
        plot,x_plot(0:xmui),yr=[-0.1*h,h*1.1],xr=[0,150],$
             ys=1,psym=-6,syms=.2,$
             title='neutroni n:o '+string(i),$
             xtitle='vuorovaikutus n:o',ytitle='x'
        oplot,x_plot*0,lines=1
        oplot,x_plot*0+h,lines=1
     endif
     oplot,x_plot(0:xmui<100),col=i+2<30

     if(n le 20) then begin
        xyouts,110,h-h/20.*(i+1),x_plot(1),col=i+2
     endif
     vast=''
     read,vast
  endif



  endfor

;-----------------------------------------------------
;tulosten kokoaminen


  p_lap=w_lap/n
  p_abs=w_abs/n
  p_hei=w_hei/n

  sqrtn=sqrt(1.*n)


  dp_lap=sqrt(w_lap2/n-(w_lap/n)^2)/sqrtn
  dp_abs=sqrt(w_abs2/n-(w_abs/n)^2)/sqrtn
  dp_hei=sqrt(w_hei2/n-(w_hei/n)^2)/sqrtn

;-----------------------------------------------------
  if(keyword_set(print)) then begin
     print,'H,Alb,n',h,alb,n
     print,'plap,pabs,phei',p_lap,p_abs,p_hei
     print,'dlap,dabs,dhei',dp_lap,dp_abs,dp_hei
  endif



seed0=seed
end
